/**
 * Copyright 2016 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import test from 'tape';
import td from 'testdouble';

import {captureHandlers} from '../helpers/foundation';
import {createMockRaf} from '../helpers/raf';
import {setupFoundationTest} from '../helpers/setup';
import MDCTemporaryDrawerFoundation from '../../../packages/mdc-drawer/temporary/foundation';
import {cssClasses, strings, numbers} from '../../../packages/mdc-drawer/temporary/constants';

function setupTest() {
  const {foundation, mockAdapter} = setupFoundationTest(MDCTemporaryDrawerFoundation);
  td.when(mockAdapter.hasClass('mdc-temporary-drawer')).thenReturn(true);
  td.when(mockAdapter.hasNecessaryDom()).thenReturn(true);
  return {foundation, mockAdapter};
}

test('exports strings', (t) => {
  t.deepEqual(MDCTemporaryDrawerFoundation.strings, strings);
  t.end();
});

test('exports cssClasses', (t) => {
  t.deepEqual(MDCTemporaryDrawerFoundation.cssClasses, cssClasses);
  t.end();
});

test('exports numbers', (t) => {
  t.deepEqual(MDCTemporaryDrawerFoundation.numbers, numbers);
  t.end();
});

test('defaultAdapter returns a complete adapter implementation', (t) => {
  const {defaultAdapter} = MDCTemporaryDrawerFoundation;
  const methods = Object.keys(defaultAdapter).filter((k) => typeof defaultAdapter[k] === 'function');

  t.equal(methods.length, Object.keys(defaultAdapter).length, 'Every adapter key must be a function');
  t.deepEqual(methods, [
    'addClass', 'removeClass', 'hasClass', 'hasNecessaryDom', 'registerInteractionHandler',
    'deregisterInteractionHandler', 'registerDrawerInteractionHandler', 'deregisterDrawerInteractionHandler',
    'registerTransitionEndHandler', 'deregisterTransitionEndHandler', 'registerDocumentKeydownHandler',
    'deregisterDocumentKeydownHandler', 'setTranslateX', 'updateCssVariable', 'getFocusableElements',
    'saveElementTabState', 'restoreElementTabState', 'makeElementUntabbable', 'isRtl', 'getDrawerWidth', 'isDrawer',
  ]);
  // Test default methods
  methods.forEach((m) => t.doesNotThrow(defaultAdapter[m]));

  t.end();
});

test('#init throws error when the root class is not present', (t) => {
  const mockAdapter = td.object(MDCTemporaryDrawerFoundation.defaultAdapter);
  td.when(mockAdapter.hasClass('mdc-temporary-drawer')).thenReturn(false);

  const foundation = new MDCTemporaryDrawerFoundation(mockAdapter);
  t.throws(() => foundation.init());
  t.end();
});

test('#init throws error when the necessary DOM is not present', (t) => {
  const mockAdapter = td.object(MDCTemporaryDrawerFoundation.defaultAdapter);
  td.when(mockAdapter.hasClass('mdc-temporary-drawer')).thenReturn(true);
  td.when(mockAdapter.hasNecessaryDom()).thenReturn(false);

  const foundation = new MDCTemporaryDrawerFoundation(mockAdapter);
  t.throws(() => foundation.init());
  t.end();
});

test('#init calls component and drawer event registrations', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const {isA} = td.matchers;

  foundation.init();
  t.doesNotThrow(() => td.verify(mockAdapter.registerInteractionHandler('click', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.registerDrawerInteractionHandler('click', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.registerDrawerInteractionHandler('touchstart', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.registerInteractionHandler('touchmove', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.registerInteractionHandler('touchend', isA(Function))));
  t.end();
});

test('#destroy calls component and drawer event deregistrations', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const {isA} = td.matchers;

  foundation.init();
  foundation.destroy();
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterInteractionHandler('click', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterDrawerInteractionHandler('click', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterDrawerInteractionHandler('touchstart', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterInteractionHandler('touchmove', isA(Function))));
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterInteractionHandler('touchend', isA(Function))));
  t.end();
});

test('#destroy ensures any currently attached document keydown handler is cleaned up', (t) => {
  const {foundation, mockAdapter} = setupTest();
  foundation.init();
  foundation.destroy();
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterDocumentKeydownHandler(td.matchers.isA(Function))));
  t.end();
});

test('#open adds the animation class to start an animation', (t) => {
  const {foundation, mockAdapter} = setupTest();

  foundation.open();
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--animating')));
  t.end();
});

test('#open adds the open class to show the open drawer', (t) => {
  const {foundation, mockAdapter} = setupTest();

  foundation.open();
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--open')));
  t.end();
});

test('#open makes elements tabbable again', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(false);
  td.when(mockAdapter.getFocusableElements()).thenReturn(['foo', 'bar']);

  foundation.init();
  foundation.open();
  t.doesNotThrow(() => td.verify(mockAdapter.restoreElementTabState('foo')));
  t.doesNotThrow(() => td.verify(mockAdapter.restoreElementTabState('bar')));
  t.end();
});

test('#close adds the animation class to start an animation', (t) => {
  const {foundation, mockAdapter} = setupTest();

  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--animating')));
  t.end();
});

test('#close removes the open class to show the closed drawer', (t) => {
  const {foundation, mockAdapter} = setupTest();

  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open')));
  t.end();
});

test('#close saves tab state and makes elements untabbable', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getFocusableElements()).thenReturn(['foo', 'bar']);

  foundation.init();
  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.saveElementTabState('foo')));
  t.doesNotThrow(() => td.verify(mockAdapter.makeElementUntabbable('foo')));
  t.doesNotThrow(() => td.verify(mockAdapter.saveElementTabState('bar')));
  t.doesNotThrow(() => td.verify(mockAdapter.makeElementUntabbable('bar')));
  t.end();
});

test('#close works when there are no elements to make untabbable', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getFocusableElements()).thenReturn(null);
  foundation.init();
  t.doesNotThrow(() => foundation.close());
  t.end();
});

test('#close cleans up the document keydown handler', (t) => {
  const {foundation, mockAdapter} = setupTest();
  foundation.init();
  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterDocumentKeydownHandler(td.matchers.isA(Function))));
  t.end();
});

test('#isOpen returns true when the drawer is open', (t) => {
  const {foundation} = setupTest();

  foundation.open();
  t.true(foundation.isOpen());
  t.end();
});

test('#isOpen returns false when the drawer is closed', (t) => {
  const {foundation} = setupTest();

  foundation.close();
  t.false(foundation.isOpen());
  t.end();
});

test('#isOpen returns true when the drawer is initiated with the open class present', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);

  foundation.init();
  t.true(foundation.isOpen());
  t.end();
});

test('#isOpen returns false when the drawer is initiated without the open class present', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(false);

  foundation.init();
  t.false(foundation.isOpen());
  t.end();
});

test('on touch start updates the drawer to the touch target coordinates', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const handlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  handlers.touchstart({
    touches: [{pageX: 50}],
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1)));

  raf.restore();
  t.end();
});

test('on touch start does not update the drawer when drawer not open', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const handlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(false);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  handlers.touchstart({
    touches: [{pageX: 50}],
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0), {times: 0}));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1), {times: 0}));

  raf.restore();
  t.end();
});

test('on touch start works for pointer events', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const handlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  handlers.touchstart({
    pointerType: 'touch',
    pageX: 50,
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1)));

  raf.restore();
  t.end();
});

test('on touch start does not update the drawer when pointertype != touch', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const handlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  handlers.touchstart({
    pointerType: 'not touch',
    pageX: 50,
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0), {times: 0}));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1), {times: 0}));

  raf.restore();
  t.end();
});

test('on touch move updates currentX causing the drawer to update', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 490}],
    preventDefault: () => {},
  });
  raf.flush();

  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(-10)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(0.98)));

  handlers.touchmove({
    touches: [{pageX: 495}],
    preventDefault: () => {},
  });
  raf.flush();

  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(-5)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(0.99)));
  raf.restore();
  t.end();
});

test('on touch move does not allow the drawer to move past its width', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 510}],
    preventDefault: () => {},
  });
  raf.flush();

  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1)));
  raf.restore();
  t.end();
});

test('on touch move does not allow the drawer to move past its width in RTL', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  td.when(mockAdapter.isRtl()).thenReturn(true);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 490}],
    preventDefault: () => {},
  });
  raf.flush();

  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(0)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(1)));
  raf.restore();
  t.end();
});

test('on touch move works for pointer events', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    pointerType: 'touch',
    pageX: 500,
  });
  raf.flush();

  handlers.touchmove({
    pointerType: 'touch',
    pageX: 490,
    preventDefault: () => {},
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(-10)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(0.98)));

  raf.restore();
  t.end();
});

test('on touch move does not update the drawer when pointertype != touch', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    pointerType: 'touch',
    pageX: 500,
  });
  raf.flush();

  handlers.touchmove({
    pointerType: 'not touch',
    pageX: 490,
  });
  raf.flush();
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(-10), {times: 0}));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable(0.98), {times: 0}));

  raf.restore();
  t.end();
});

test('on touch end resets touch update styles', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchend({});
  t.doesNotThrow(() => td.verify(mockAdapter.setTranslateX(null)));
  t.doesNotThrow(() => td.verify(mockAdapter.updateCssVariable('')));
  raf.restore();
  t.end();
});

test('on touch end closes the drawer if moved more than 50%', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 100}],
    preventDefault: () => {},
  });

  handlers.touchend({});
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open')));
  raf.restore();
  t.end();
});

test('on touch end keeps the drawer open if moved less than 50%', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 500}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 300}],
    preventDefault: () => {},
  });

  handlers.touchend({});
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--open')));
  raf.restore();
  t.end();
});

test('on touch end closes the drawer if moved more than 50% in RTL', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  td.when(mockAdapter.isRtl()).thenReturn(true);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 100}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 500}],
    preventDefault: () => {},
  });

  handlers.touchend({});
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open')));
  raf.restore();
  t.end();
});

test('on touch end keeps the drawer open if moved less than 50% in RTL', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  td.when(mockAdapter.isRtl()).thenReturn(true);
  foundation.init();

  drawerHandlers.touchstart({
    touches: [{pageX: 300}],
  });
  raf.flush();

  handlers.touchmove({
    touches: [{pageX: 500}],
    preventDefault: () => {},
  });

  handlers.touchend({});
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--open')));
  raf.restore();
  t.end();
});

test('on touch end works with pointer events', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    pointerType: 'touch',
    pageX: 500,
  });
  raf.flush();

  handlers.touchmove({
    pointerType: 'touch',
    pageX: 300,
    preventDefault: () => {},
  });

  handlers.touchend({
    pointerType: 'touch',
  });
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--open')));
  raf.restore();
  t.end();
});

test('on touch does nothing for non touch pointer events', (t) => {
  const {foundation, mockAdapter} = setupTest();
  const drawerHandlers = captureHandlers(mockAdapter, 'registerDrawerInteractionHandler');
  const handlers = captureHandlers(mockAdapter, 'registerInteractionHandler');
  const raf = createMockRaf();
  td.when(mockAdapter.hasClass('mdc-temporary-drawer--open')).thenReturn(true);
  td.when(mockAdapter.getDrawerWidth()).thenReturn(500);
  foundation.init();

  drawerHandlers.touchstart({
    pointerType: 'touch',
    pageX: 500,
  });
  raf.flush();

  handlers.touchmove({
    pointerType: 'touch',
    pageX: 300,
    preventDefault: () => {},
  });

  handlers.touchend({
    pointerType: 'not touch',
  });
  t.doesNotThrow(() => td.verify(mockAdapter.addClass('mdc-temporary-drawer--open'), {times: 0}));
  raf.restore();
  t.end();
});

test('on document keydown closes the drawer via the escape key', (t) => {
  const {foundation, mockAdapter} = setupTest();
  let keydown;
  td.when(mockAdapter.registerDocumentKeydownHandler(td.matchers.isA(Function))).thenDo((handler) => {
    keydown = handler;
  });
  foundation.init();
  foundation.open();

  keydown({
    key: 'Escape',
  });
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open')));
  t.end();
});

test('on document keydown closes the drawer via the escape keyCode when key prop not available', (t) => {
  const {foundation, mockAdapter} = setupTest();
  let keydown;
  td.when(mockAdapter.registerDocumentKeydownHandler(td.matchers.isA(Function))).thenDo((handler) => {
    keydown = handler;
  });
  foundation.init();
  foundation.open();

  keydown({
    keyCode: 27,
  });
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open')));
  t.end();
});

test('on document keydown does nothing when key present but not "Escape"', (t) => {
  const {foundation, mockAdapter} = setupTest();
  let keydown;
  td.when(mockAdapter.registerDocumentKeydownHandler(td.matchers.isA(Function))).thenDo((handler) => {
    keydown = handler;
  });
  foundation.init();
  foundation.open();

  keydown({
    key: 'Enter',
  });
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open'), {times: 0}));
  t.end();
});

test('on document keydown does nothing when key prop not present and keyCode is not 27', (t) => {
  const {foundation, mockAdapter} = setupTest();
  let keydown;
  td.when(mockAdapter.registerDocumentKeydownHandler(td.matchers.isA(Function))).thenDo((handler) => {
    keydown = handler;
  });
  foundation.init();
  foundation.open();

  keydown({
    keyCode: 32,
  });
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--open'), {times: 0}));
  t.end();
});

test('on document keydown does nothing if drawer is not opened', (t) => {
  const {foundation, mockAdapter} = setupTest();
  let keydown;
  td.when(mockAdapter.registerDocumentKeydownHandler(td.matchers.isA(Function))).thenDo((handler) => {
    keydown = handler;
  });
  foundation.init();

  t.true(keydown === undefined);
  t.end();
});

test('should clean up transition handlers after drawer close', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.isDrawer(td.matchers.isA(Object))).thenReturn(true);
  td.when(mockAdapter.registerTransitionEndHandler(td.callback)).thenCallback({target: {}});
  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterTransitionEndHandler(td.matchers.isA(Function))));
  t.end();
});

test('should not trigger bug #67', (t) => {
  const {foundation, mockAdapter} = setupTest();
  td.when(mockAdapter.isDrawer(td.matchers.isA(Object))).thenReturn(false);
  td.when(mockAdapter.registerTransitionEndHandler(td.callback)).thenCallback({target: null});
  foundation.close();
  t.doesNotThrow(() => td.verify(mockAdapter.removeClass('mdc-temporary-drawer--animating'), {times: 0}));
  t.doesNotThrow(() => td.verify(mockAdapter.deregisterTransitionEndHandler(td.matchers.isA(Function)), {times: 0}));
  t.end();
});
